@@ -11,6 +11,7 @@ from dateutil.parser import parse
 from dateutil.relativedelta import relativedelta
 
 from datetimerange import DateTimeRange
+from datetimerange._core import _normalize_datetime_value
 
 
 TIMEZONE = "+0900"
@@ -30,38 +31,42 @@ def setup_module(module):
 
 @pytest.fixture
 def datetimerange_normal():
-    value = DateTimeRange(TEST_START_DATETIME, TEST_END_DATETIME)
-    value.start_time_format = ISO_TIME_FORMAT
-    value.end_time_format = ISO_TIME_FORMAT
-
-    return value
+    return DateTimeRange(
+        TEST_START_DATETIME,
+        TEST_END_DATETIME,
+        start_time_format=ISO_TIME_FORMAT,
+        end_time_format=ISO_TIME_FORMAT,
+    )
 
 
 @pytest.fixture
 def datetimerange_inversion():
-    value = DateTimeRange(TEST_END_DATETIME, TEST_START_DATETIME)
-    value.start_time_format = ISO_TIME_FORMAT
-    value.end_time_format = ISO_TIME_FORMAT
-
-    return value
+    return DateTimeRange(
+        TEST_END_DATETIME,
+        TEST_START_DATETIME,
+        start_time_format=ISO_TIME_FORMAT,
+        end_time_format=ISO_TIME_FORMAT,
+    )
 
 
 @pytest.fixture
 def datetimerange_null():
-    value = DateTimeRange(None, None)
-    value.time_format = None
-    value.end_time_format = None
-
-    return value
+    return DateTimeRange(
+        None,
+        None,
+        start_time_format=None,
+        end_time_format=None,
+    )
 
 
 @pytest.fixture
 def datetimerange_null_start():
-    value = DateTimeRange(None, TEST_END_DATETIME)
-    value.time_format = None
-    value.end_time_format = ISO_TIME_FORMAT
-
-    return value
+    return DateTimeRange(
+        None,
+        TEST_END_DATETIME,
+        start_time_format=None,
+        end_time_format=ISO_TIME_FORMAT,
+    )
 
 
 class TestDateTimeRange_repr:
@@ -77,6 +82,15 @@ class TestDateTimeRange_repr:
                 False,
                 "2015-03-22T10:00:00+0900 - 2015-03-22T10:10:00+0900",
             ],
+            [
+                TEST_START_DATETIME,
+                None,
+                TEST_END_DATETIME,
+                None,
+                " - ",
+                False,
+                "2015-03-22T10:00:00+0900 - 2015-03-22T10:10:00+0900",
+            ],
             [
                 "2015-03-22T09:00:00+0900",
                 ISO_TIME_FORMAT,
@@ -194,24 +208,6 @@ class TestDateTimeRange_repr:
     @pytest.mark.parametrize(
         ["start", "start_format", "end", "end_format", "separator", "is_output_elapse", "expected"],
         [
-            [
-                TEST_START_DATETIME,
-                None,
-                TEST_END_DATETIME,
-                ISO_TIME_FORMAT,
-                " - ",
-                False,
-                TypeError,
-            ],
-            [
-                TEST_START_DATETIME,
-                ISO_TIME_FORMAT,
-                TEST_END_DATETIME,
-                None,
-                " - ",
-                False,
-                TypeError,
-            ],
             [
                 TEST_START_DATETIME,
                 ISO_TIME_FORMAT,
@@ -336,7 +332,11 @@ class TestDateTimeRange_add:
 
     @pytest.mark.parametrize(
         ["value", "expected"],
-        [["2015-03-22T10:10:00+0900", TypeError], [1, TypeError], [None, TypeError]],
+        [
+            ["2015-03-22T10:10:00+0900", TypeError],
+            [1, TypeError],
+            [None, TypeError],
+        ],
     )
     def test_exception(self, datetimerange_normal, value, expected):
         with pytest.raises(TypeError):
@@ -524,6 +524,39 @@ class TestDateTimeRange_is_set:
         assert value.is_set() == expected
 
 
+class TestDateTimeRange_is_time_inversion:
+    @pytest.mark.parametrize(
+        ["value", "expected"],
+        [
+            [DateTimeRange(TEST_START_DATETIME, TEST_END_DATETIME), False],
+            [DateTimeRange(TEST_START_DATETIME, TEST_START_DATETIME), False],
+            [DateTimeRange(TEST_END_DATETIME, TEST_START_DATETIME), True],
+        ],
+    )
+    def test_normal(self, value, expected):
+        assert value.is_time_inversion() == expected
+
+    def test_allow_timezone_mismatch(self):
+        dtr = DateTimeRange("2022-10-29 00:00:00+02:00", "2022-10-31 00:00:00+01:00")
+
+        assert not dtr.is_time_inversion(allow_timezone_mismatch=True)
+
+        with pytest.raises(ValueError):
+            dtr.is_time_inversion(allow_timezone_mismatch=False)
+
+    @pytest.mark.parametrize(
+        ["value"],
+        [
+            [DateTimeRange(None, None)],
+            [DateTimeRange(None, TEST_END_DATETIME)],
+            [DateTimeRange(TEST_START_DATETIME, None)],
+        ],
+    )
+    def test_exception(self, value):
+        with pytest.raises(ValueError):
+            value.is_time_inversion()
+
+
 class TestDateTimeRange_validate_time_inversion:
     @pytest.mark.parametrize(
         ["value"],
@@ -551,6 +584,14 @@ class TestDateTimeRange_validate_time_inversion:
         with pytest.raises(TypeError):
             value.validate_time_inversion()
 
+    def test_allow_timezone_mismatch(self):
+        dtr = DateTimeRange("2022-10-29 00:00:00+02:00", "2022-10-31 00:00:00+01:00")
+
+        assert not dtr.is_time_inversion(allow_timezone_mismatch=True)
+
+        with pytest.raises(ValueError):
+            dtr.validate_time_inversion(allow_timezone_mismatch=False)
+
 
 class TestDateTimeRange_is_valid_timerange:
     @pytest.mark.parametrize(
@@ -619,7 +660,7 @@ class TestDateTimeRange_range:
                 ],
             ],
             [
-                DateTimeRange(date(2015, 3, 23), date(2015, 3, 26)),
+                DateTimeRange(datetime(2015, 3, 23), datetime(2015, 3, 26)),
                 relativedelta(days=+1),
                 [
                     datetime(2015, 3, 23, 0, 0, 0),
@@ -636,6 +677,19 @@ class TestDateTimeRange_range:
         for value_item, expected_item in zip(results, expected):
             assert value_item == expected_item
 
+    def test_normal_dst(self):
+        dtr = DateTimeRange("2022-10-29 00:00:00+02:00", "2022-10-31 00:00:00+01:00")
+        results = list(dtr.range(timedelta(days=1)))
+        timezone = pytz.timezone("Africa/Tripoli")
+        expected = [
+            _normalize_datetime_value(datetime(2022, 10, 29, 0, 0, 0), timezone=timezone),
+            _normalize_datetime_value(datetime(2022, 10, 30, 0, 0, 0), timezone=timezone),
+            _normalize_datetime_value(datetime(2022, 10, 31, 0, 0, 0), timezone=timezone),
+        ]
+        assert len(results) == len(expected)
+        for value_item, expected_item in zip(results, expected):
+            assert value_item == expected_item
+
     @pytest.mark.parametrize(
         ["value", "step", "expected"],
         [
@@ -749,6 +803,13 @@ class TestDateTimeRange_is_intersection:
                 timedelta(seconds=1),
                 True,
             ],
+            [
+                # https://github.com/thombashi/DateTimeRange/issues/48
+                DateTimeRange("2015-03-22T10:00:00+0900", "2015-03-22T10:10:00+0900"),
+                DateTimeRange("2015-03-23T10:05:00+0900", "2015-03-23T10:15:00+0900"),
+                timedelta(seconds=1),
+                False,
+            ],
         ],
     )
     def test_normal_w_intersection_threshold(self, lhs, rhs, threshold, expected):
@@ -1026,6 +1087,24 @@ class TestDateTimeRange_set_time_range:
         dtr.set_time_range(start, end)
         assert dtr == expected
 
+    def test_normal_timezone(self):
+        dtr_utc_lhs = DateTimeRange(TEST_START_DATETIME, TEST_END_DATETIME, timezone=pytz.UTC)
+        dtr_utc_rhs = DateTimeRange()
+        dtr_utc_rhs.set_time_range(TEST_START_DATETIME, TEST_END_DATETIME, timezone=pytz.UTC)
+        assert dtr_utc_lhs == dtr_utc_rhs
+
+        dtr_tokyo_lhs = DateTimeRange(TEST_START_DATETIME, TEST_END_DATETIME, timezone=pytz.timezone("Asia/Tokyo"))
+        dtr_tokyo_rhs = DateTimeRange()
+        dtr_tokyo_rhs.set_time_range(TEST_START_DATETIME, TEST_END_DATETIME, timezone=pytz.timezone("Asia/Tokyo"))
+        assert dtr_tokyo_lhs == dtr_tokyo_rhs
+
+        assert dtr_utc_lhs == dtr_tokyo_lhs
+
+    def test_normal_replace_timezone(self):
+        dtr_lhs = DateTimeRange(START_DATETIME_TEXT, START_DATETIME_TEXT, timezone=pytz.UTC)
+        dtr_rhs = DateTimeRange(START_DATETIME_TEXT, START_DATETIME_TEXT, timezone=pytz.timezone("Asia/Tokyo"))
+        assert dtr_lhs != dtr_rhs
+
     @pytest.mark.parametrize(
         ["start", "end", "expected"],
         [
@@ -1298,6 +1377,10 @@ class TestDateTimeRange_from_range_text:
         assert dtr.start_time_format == r"%Y-%m-%dT%H:%M:%S%z"
         assert dtr.end_time_format == r"%Y-%m-%dT%H:%M:%S%z"
 
+    def test_normal_tz(self):
+        dtr = DateTimeRange.from_range_text(f"{START_DATETIME_TEXT} - {END_DATETIME_TEXT}", timezone=pytz.utc)
+        assert dtr.timezone == pytz.utc
+
     @pytest.mark.parametrize(
         ["value", "time_format", "expected"],
         [
